##############################################################################
#
# Copyright (c) 2001, 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

# allows integration of archive files into Zope and access the contents of them
# like objects in a folder by Joachim Bauch
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.Image import File
from OFS.content_types import guess_content_type
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from ZipArchive import ZipArchive
from ArchiveImage import ArchiveFile, ArchiveImage

_marker=[]

class ArchiveFolder(Folder):

    meta_type = 'Archive Folder'
    
    security = ClassSecurityInfo()

    manage_options = (Folder.manage_options[0], 
                      {'label':'Upload', 'action':'manage_uploadFileForm', },
                     )+Folder.manage_options[1:]

    manage_uploadFileForm = PageTemplateFile('zmi/manage_uploadFile.pt', globals())

    def filtered_meta_types(self, user=None):
        types = Folder.filtered_meta_types(self)
        result = []
        for t in types:
            if t['name'] in ['Image', 'File', ]:
                result.append(t)
        return result

    def __init__(self, id, title=None, archiveType='zip'):
        self.id = str(id)
        self.title = title
        if not archiveType in ['zip', ]:
            raise ValueError, 'Archive type "%s" is currently not supported.' % (archiveType)
        self.archiveType = archiveType
        self._archive = None
        self._archiveFiles = {}

    def objectFactory(self, name, params):
        ct, enc = guess_content_type(name=name)
        if ct.startswith('image/'):
            # read 1k bytes to determine image size
            data = self._archive.getFile(name, cache=0)[:1024]
            return ArchiveImage(name, params, ct, data=data)

        return ArchiveFile(name, params, ct)

    def manage_uploadFile(self, file, REQUEST=None):
        """ stores an archive file as folder """
        self._archive = ZipArchive(file)
        files = self._archive.getArchiveContents().items()
        self._archiveFiles = {}
        for name, value in files:
            if '/' in name:
                raise ValueError, 'Archives with files in folders are not supported'
            self._archiveFiles[name] = self.objectFactory(name, value)
        if REQUEST:
            return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_main?manage_tabs_message=File+stored.')        

    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
        v=self._checkId(id)
        if v is not None: id=v
        try:    t=object.meta_type
        except: t=None

        self.setFile(id, object.data)
        object.manage_afterAdd(object, self)
        return id

    def _getOb(self, id, default=_marker):
        # FIXME: what we really need to do here is ensure that only
        # sub-items are returned. That could have a measurable hit
        # on performance as things are currently implemented, so for
        # the moment we just make sure not to expose private attrs.
        if id[:1] != '_' and self._archiveFiles.has_key(id):
            return self._archiveFiles[id].__of__(self)
        if default is _marker:
            raise AttributeError, id
        return default

    def setFile(self, filename, data):
        if self._archive is None:
            # create new empty archive
            self._archive = ZipArchive('')
        info = self._archive.setFile(filename, data)
        object = self.objectFactory(filename, info)
        tmp = self._archiveFiles
        tmp[filename] = object
        self._archiveFiles = tmp

    def __getattr__(self, name):
        if self._archiveFiles.has_key(name):
            # return archive file as attribute of our folder
            return self._archiveFiles[name].__of__(self)
        elif self.__dict__.has_key(name):
            # normal attribute
            return self.__dict__[name]
        else:
            return Folder.__getattr__(self, name)

    def getFile(self, name):
        return self._archive.getFile(name)

    def objectItems(self):
        if self._archive is None:
            return []
        
        result = []
        for name, value in self._archiveFiles.items():
            value = value.__of__(self)
            result.append((name, value))
        return result

    def index_html(self, REQUEST, RESPONSE):
        """ return archive contents """
        if self._archive is None:
            return None
        RESPONSE.setHeader('Content-Disposition', 'attachment;filename=%s' % (self.id))
        return self._archive(REQUEST, RESPONSE)
        
manage_addArchiveFolderForm = PageTemplateFile('zmi/manage_addArchiveFolder.pt', globals(), __name__ = 'manage_addArchiveFolderForm')

def manage_addArchiveFolder(self, id, title=None, archiveType='zip', REQUEST=None):
    """ adds a new ArchiveFolder to the current folder """
    folder = ArchiveFolder(id, title, archiveType)
    self._setObject(id, folder)
    folder = getattr(self, id)
    if REQUEST is not None:
        return REQUEST.RESPONSE.redirect(folder.absolute_url() + '/manage_main?update_menu=1')
