##############################################################################
#
# Copyright (c) 2001, 2002, 2003 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

# allows integration of archive files into Zope and access the contents of them
# like objects in a folder by Joachim Bauch
import StringIO, zipfile
from OFS.Folder import Folder
from DateTime import DateTime

class ZipArchive:

    def __init__(self, file, cache=1):
        self._cache = cache
        if type(file) == type(''):
            self._file = StringIO.StringIO(file)
        elif hasattr(file, 'read'):
            self._file = StringIO.StringIO(file.read())
        else:
            self._file = file
        zip = zipfile.ZipFile(self._file)
        self._parseFile(zip)
        zip.close()
        if self._cache:
            self._archiveCache = {}

    def _parseFile(self, zip):
        files = zip.infolist()
        self._archiveContents = {}
        for file in files:
            fileinfo = {}
            fileinfo['file_date'] = file.date_time
            fileinfo['file_size'] = file.file_size
            # internal attributes
            fileinfo['offset'] = file.file_offset
            fileinfo['compressed_size'] = file.compress_size
            self._archiveContents[file.filename] = fileinfo

    def getFilenames(self):
        return self._archiveContents.keys()
        
    def getArchiveContents(self):
        return self._archiveContents

    def getFile(self, filename, cache=1):
        if self._cache and self._archiveCache.has_key(filename):
            return self._archiveCache[filename]
        
        if not self._archiveContents.has_key(filename):
            raise 'ArchiveError', "The file doesn't exist."
        
        if not self._archiveContents[filename]['file_size']:
            return ''
        
        zip = zipfile.ZipFile(self._file)
        data = zip.read(filename)
        zip.close()
        if cache and self._cache:
            tmp = self._archiveCache
            tmp[filename] = data
            self._archiveCache = tmp
        return data
        
    def setFile(self, filename, data):
        y, m, d, h, n, s, z = DateTime().parts()
        dt = tuple(map(lambda x: int(x), (y, m, d, h, n, s)))
        if not data:
            # create dummy file, but don't write to zipfile
            m = self._archiveContents
            m[filename] = {'file_date': dt, 'file_size': 0}
            self._archiveContents = m
            return self._archiveContents[filename]
        
        zip = zipfile.ZipFile(self._file, 'a', zipfile.ZIP_DEFLATED)
        info = zipfile.ZipInfo(filename, dt)
        info.file_size = len(data)
        info.compress_type = zipfile.ZIP_DEFLATED
        zip.writestr(info, str(data))
        self._parseFile(zip)
        zip.close()
        return self._archiveContents[filename]
        
    def __call__(self, REQUEST, RESPONSE):
        RESPONSE.setHeader('Content-Type', 'application/x-zip-compressed')
        return self._file.getvalue()